
CREATE OR REPLACE PROCEDURE maxdata.p_version_copy_pog
        (in_jobID NUMBER)
AS

/*
Revision 1.1.2.2.2.3  2012/07/24 22:16:51  gacard
FIXID S0831894: NF: Option to include unplaced products upon planogram copy

Revision 1.1.2.1.2.3  2010/03/03 20:03:59  gacard
FIXID S0618292: NF: Affinity Map Manager needed for retrieval and edit of composite planograms

Revision 1.1.2.1.2.2  2010/02/22 22:16:49  gacard
FIXID S0590015: NF: Planogram Maintenance needs option for handling signage

Revision 1.1.2.1.2.1  2009/07/21 19:13:26  makirk
FIXID S0576852: No comment given. See defect documentation.

Revision 1.1.2.1  2008/05/12 20:53:03  vejang
FIXID : 511 Scripts added

Revision 1.1  2008/05/09 20:10:56  makirk
Renamed to 6.x standard
Added for rename from 060_511_140_p_version_copy_pog.sql.
See originally named file for history prior to the rename.

Revision 1.4  2008/05/07 20:31:35  makirk
Removed "order by" from CTAS stmt as it is unnecessary

Revision 1.3  2008/05/06 18:14:17  makirk
Added "where exists" clause to correlated updates

Revision 1.2  2008/04/18 20:45:50  makirk
Added peg_link_id update

Revision 1.1  2008/04/17 20:50:08  makirk
Fix for S0499002


-- MCK, 04/03/08, Fix for S0499002

History
-------
02/21/2014 - increase size of lv7loc_userid variables to match lv7loc.lv7loc_userid
02/26/2010 - fix for lv7loc_userid that contains single quote
02/17/2010 - don't call p_get_next_key with count of zero
07/16/2009 - removed check for record_type = 'M' in order to copy composite planogram
04/03/2008 - Added for Planogram Maintenance

Usage
-----
In order to use this proc there are 2 tables that are populated by the calling code as follows:
	maxdata.t_pogs_to_copy that has columns job_id, lv7loc_id, new_lv7loc_id
	maxdata.t_assignments_to_copy that has columns job_id, lv7loc_id,new_lv7loc_id, pog_master_id, new_pog_master_id, effective_date
There are 2 tables because there could be some planograms without any assignments and some planograms will have multiple assignments.
The procedure takes one argument, in_jobID, which is used to indicate which records in the input tables are to be processed.
The procedure will make copies of all records all the corresponding records in lv7,lv8,lv9,lv10 loc and pogmaster tables.
The procedure will update the input table with the new lv7loc_ids and new pog_master_ids.

NOTE: Dynamic SQL must be used because tables used in this proc are not permanent and therefore all SQL stmts
      using them will fail to compile
*/

-- Template vars (error handling, tracking, sql text)

n_sqlnum		NUMBER(10)	:= 1000;
t_proc_name		VARCHAR2(30)	:= 'p_version_copy_pog';
t_text			VARCHAR2(255);
t_error_msg		VARCHAR2(255);
t_copy_unplaced		INTEGER;	--1=copy unplaced products, 0=don't copy unplaced products
v_lv7loc_id		NUMBER(10);
v_to_7mlv7loc_id	NUMBER(10);
v_to_pog_master_id	NUMBER(10)	:= 0;
v_to_8llv8loc_id	NUMBER(10)	:= 0;
v_to_9llv9loc_id	NUMBER(10)	:= 0;
v_to_10llv10loc_id	NUMBER(10)	:= 0;
v_lv10locCounter	NUMBER(10);
v_lv7count		NUMBER(10);
v_lv8count		NUMBER(10);
v_lv9count		NUMBER(10);
v_lv10count		NUMBER(10);
v_pogcount		NUMBER(10);
v_sql			VARCHAR2(3000);

v_lv7loc_userid         VARCHAR2(80);
v_currlv7loc_userid	VARCHAR2(80);
v_quoted_currlv7loc_userid	VARCHAR2(160);
v_prevlv7loc_userid	VARCHAR2(80);

v_counter		NUMBER(10);
v_name			VARCHAR2(128);
v_version_id            NUMBER(10);
t_cur_stmt 		VARCHAR2(1000);
v_sessionid		NUMBER(10);

TYPE TabCurTyp IS REF CURSOR;
c_colname TabCurTyp;

CURSOR c_lv7loc IS
	SELECT DISTINCT
	       F.version_id,
	       D.lv7loc_userid,
	       D.lv7loc_id
	  FROM (SELECT (ROWNUM)-1 AS Newlv7loc_id,
		       A.*
		  FROM maxdata.lv7loc A,
		       maxdata.t_pogs_to_copy B
		 WHERE A.lv7loc_id   = B.lv7loc_id
		   AND B.job_id      = in_jobID
		 ORDER BY lv7loc_userid) B,
		maxdata.lv7loc D,
	       (SELECT MAX(version_id) version_id,
		       lv7loc_userid
		  FROM maxdata.lv7loc
		 GROUP BY lv7loc_userid) F
	 WHERE D.lv7loc_id     = B.lv7loc_id
	   AND D.lv7loc_userid = F.lv7loc_userid
	 ORDER BY D.lv7loc_userid,
	          D.lv7loc_id;
BEGIN

-- Log start
n_sqlnum := 1010;
t_text   := 'Started';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

-- Check to see if job_id is valid
SELECT COUNT(*) INTO v_counter FROM maxdata.t_pogs_to_copy WHERE job_id = in_jobID;

IF v_counter = 0 THEN
	t_error_msg := 'Job ID '||in_jobid||' was not found in maxdata.t_pogs_to_copy';
        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        raise_application_error (-20001, RTRIM(t_error_msg));
END IF;

--dbms_output.put_line(t_text);

-- Get the unique userid for protecting tables from other users
SELECT userenv('sessionid') INTO v_sessionid FROM dual;

-------------------------------------------------------------------
-- Get the lv7loc records to be copied for the pog
-------------------------------------------------------------------
t_text := 'Building maxdata.lv7loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(t_text);

n_sqlnum := 1020;
v_sql := 'CREATE TABLE maxdata.lv7loc_to_be_copied_'||v_sessionid||' AS '||
	 'SELECT (ROWNUM)-1 AS Newlv7loc_id, '||
	 '       A.* '||
	 '  FROM maxdata.lv7loc A, '||
	 '       maxdata.t_pogs_to_copy B '||
	 ' WHERE A.lv7loc_id   = B.lv7loc_id '||
	 '   AND B.job_id      = '|| in_jobID;

--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

--Getting the new keys and updating the sequence table
n_sqlnum := 1030;
v_sql := 'SELECT COUNT(*) FROM maxdata.lv7loc_to_be_copied_'||v_sessionid;

EXECUTE IMMEDIATE v_sql INTO v_lv7count;

IF v_lv7count != 0 THEN
    maxapp.p_get_next_key(7, 2, v_lv7count, v_to_7mlv7loc_id, t_error_msg);

    IF t_error_msg IS NOT NULL THEN
        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        raise_application_error (-20001, RTRIM(t_error_msg));
    END IF;
END IF;

n_sqlnum := 1040;
t_text   := 'Copying ' || v_lv7count || ' planograms(s)';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

-------------------------------------------------------------------
-- Get the lv8loc records to be copied for the lv7loc records
-------------------------------------------------------------------
t_text := 'Building maxdata.lv8loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(t_text);

n_sqlnum := 1050;
v_sql := 'CREATE TABLE maxdata.lv8loc_to_be_copied_'||v_sessionid||' AS '||
	 'SELECT (ROWNUM)-1 AS Newlv8loc_id, '||
         '       A.*, '||
         '       B.Newlv7loc_id + '|| v_to_7mlv7loc_id ||' Newlv7loc_id '||
         '  FROM maxdata.lv8loc A, '||
         '       maxdata.lv7loc_to_be_copied_'||v_sessionid||' B '||
         ' WHERE A.lv7loc_id = B.lv7loc_id';

EXECUTE IMMEDIATE (v_sql);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('lv8loc_to_be_copied');
SELECT * FROM maxdata.lv8loc_to_be_copied
*/

--Getting the new keys and updating the sequence table
n_sqlnum := 1060;
v_sql := 'SELECT COUNT(*) FROM maxdata.lv8loc_to_be_copied_'||v_sessionid;

EXECUTE IMMEDIATE v_sql INTO v_lv8count;

n_sqlnum := 1100;

IF v_lv8count != 0 THEN
    maxapp.p_get_next_key(8, 2, v_lv8count, v_to_8llv8loc_id, t_error_msg);

    IF t_error_msg IS NOT NULL THEN
        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        raise_application_error (-20001, RTRIM(t_error_msg));
    END IF;
END IF;

-------------------------------------------------------------------
-- Get the lv9loc records to be copied for the lv8loc records
-------------------------------------------------------------------
t_text := 'Building maxdata.lv9loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(t_text);

n_sqlnum := 1110;
v_sql := 'CREATE TABLE maxdata.lv9loc_to_be_copied_'||v_sessionid||' AS '||
         'SELECT (ROWNUM)-1 Newlv9loc_id, '||
         '       A.*, '||
         '       B.Newlv7loc_id Newlv7loc_id, '||
         '       B.Newlv8loc_id + '|| v_to_8llv8loc_id ||' Newlv8loc_id '||
         '  FROM maxdata.lv9loc A, '||
         '       maxdata.lv8loc_to_be_copied_'||v_sessionid||' B '||
         ' WHERE A.lv8loc_id = B.lv8loc_id';

EXECUTE IMMEDIATE (v_sql);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('lv9loc_to_be_copied');
SELECT * FROM maxdata.lv9loc_to_be_copied
*/

--Getting the new keys and updating the sequence table
n_sqlnum := 1115;
v_sql := 'SELECT COUNT(*) FROM maxdata.lv9loc_to_be_copied_'||v_sessionid;

EXECUTE IMMEDIATE v_sql INTO v_lv9count;

IF v_lv9count != 0 THEN
    maxapp.p_get_next_key(9, 2, v_lv9count, v_to_9llv9loc_id, t_error_msg);

    IF t_error_msg IS NOT NULL THEN
        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        raise_application_error (-20001, RTRIM(t_error_msg));
    END IF;
END IF;

n_sqlnum := 1116;

SELECT TO_NUMBER(value_1) INTO t_copy_unplaced 
FROM maxapp.userpref WHERE key_1 = 'INCLUDE_UNPLACED_UPON_POG_COPY';

-------------------------------------------------------------------
-- Get the lv10loc records to be copied for the lv7loc records
-- because we want products attached to fixture level shelves as well
-- as those directly attached to the planogram.
-------------------------------------------------------------------
t_text := 'Building maxdata.lv10loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(t_text);

n_sqlnum := 1120;

IF t_copy_unplaced = 1 THEN
v_sql := 'CREATE TABLE lv10loc_to_be_copied_'||v_sessionid||' AS '||
         'SELECT (ROWNUM)-1 AS Newlv10loc_id, '||
         '       A.*, '||
         '       B.Newlv7loc_id + '|| v_to_7mlv7loc_id ||' Newlv7loc_id, '||
         '       A.lv8loc_id Newlv8loc_id, '||
         '       A.lv9loc_id Newlv9loc_id '||
         '  FROM maxdata.lv10loc A, '||
         '       maxdata.lv7loc_to_be_copied_'||v_sessionid||' B '||
         ' WHERE A.lv7loc_id = B.lv7loc_id';
ELSE
v_sql := 'CREATE TABLE lv10loc_to_be_copied_'||v_sessionid||' AS '||
         'SELECT (ROWNUM)-1 AS Newlv10loc_id, '||
         '       A.*, '||
         '       B.Newlv7loc_id + '|| v_to_7mlv7loc_id ||' Newlv7loc_id, '||
         '       A.lv8loc_id Newlv8loc_id, '||
         '       A.lv9loc_id Newlv9loc_id '||
         '  FROM maxdata.lv10loc A, '||
         '       maxdata.lv7loc_to_be_copied_'||v_sessionid||' B '||
         ' WHERE A.lv7loc_id = B.lv7loc_id  '||
         '   AND A.placed = 1';
END IF;

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1130;
v_sql := 'UPDATE maxdata.lv10loc_to_be_copied_'||v_sessionid||
         '   SET Newlv8loc_id = (SELECT L8.Newlv8loc_id + '|| v_to_8llv8loc_id ||
         '   			   FROM maxdata.lv8loc_to_be_copied_'||v_sessionid||' L8 '||
         '  			  WHERE lv10loc_to_be_copied_'||v_sessionid||'.lv8loc_id = L8.lv8loc_id), '||
         '       Newlv9loc_id = (SELECT L9.Newlv9loc_id + '|| v_to_9llv9loc_id ||
         '       		   FROM maxdata.lv9loc_to_be_copied_'||v_sessionid||' L9 '||
         '   			  WHERE lv10loc_to_be_copied_'||v_sessionid||'.lv9loc_id = L9.lv9loc_id)';

EXECUTE IMMEDIATE (v_sql);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('lv10loc_to_be_copied');
SELECT * FROM maxdata.lv10loc_to_be_copied
*/

--Getting the new keys and updating the sequence table
n_sqlnum := 1140;
v_sql := 'SELECT COUNT(*) FROM maxdata.lv10loc_to_be_copied_'||v_sessionid;

EXECUTE IMMEDIATE v_sql INTO v_lv10count;

n_sqlnum := 1150;

IF v_lv10count != 0 THEN
    maxapp.p_get_next_key(10, 2, v_lv10count, v_to_10llv10loc_id, t_error_msg);

    IF t_error_msg IS NOT NULL THEN
        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        raise_application_error (-20001, RTRIM(t_error_msg));
    END IF;
END IF;

n_sqlnum := 1200;
t_text   := 'Done gathering location records, new keys and updating sequence table';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--------------------------------------------------------------------------------
-- Get the records that need to be inserted into pogmaster
-- Taking care of assigning the copied model POG to the live location from which
-- the previous assigned POG was copied.
-- The live_lv7loc_id is being taken as is from the
-- previous pogmaster entry so, that should take care
-- of the live location assignment.
-- The new Model POG is being updated.
--------------------------------------------------------------------------------
t_text := 'Building maxdata.pm_to_be_inserted_'||v_sessionid;
--dbms_output.put_line(t_text);

n_sqlnum := 1210;
v_sql := 'CREATE TABLE maxdata.pm_to_be_inserted_'||v_sessionid||' AS '||
	 'SELECT (ROWNUM)-1 AS new_pog_master_id, '||
	 '       C.Newlv7loc_id + '|| v_to_7mlv7loc_id ||' AS new_pog_model_id, '||
	 '       A.* '||
	 '  FROM maxdata.pogmaster A '||
	 '  JOIN maxdata.t_assignments_to_copy B ON A.pog_master_id = B.pog_master_id '||
	 '  JOIN maxdata.lv7loc_to_be_copied_'||v_sessionid||' C ON B.lv7loc_id     = C.lv7loc_id '||
	 ' WHERE B.job_id = '|| in_jobID;

--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1220;
v_sql := 'UPDATE maxdata.pm_to_be_inserted_'||v_sessionid||
	 '   SET pog_planned_start = (SELECT effective_date '||
	 '				FROM maxdata.t_assignments_to_copy pa '||
	 '			       WHERE pm_to_be_inserted_'||v_sessionid||'.pog_master_id = pa.pog_master_id '||
	 '			         AND job_id = '|| in_jobID ||'), '||
	 '       pog_actual_start  = (SELECT effective_date '||
	 '				FROM maxdata.t_assignments_to_copy pa '||
	 '			       WHERE pm_to_be_inserted_'||v_sessionid||'.pog_master_id = pa.pog_master_id '||
	 '			         AND job_id = '|| in_jobID ||'), '||
	 '       pog_end_date      = NULL, '||
	 '       pog_start_cycle   = NULL, '||
	 '       pog_start_period  = NULL, '||
	 '       pog_end_cycle     = NULL, '||
	 '       pog_end_period    = NULL, '||
	 '       approval_status   = 0, '||
	 '       pog_mvmt_applied  = 0, '||
	 '       manually_approved = 0, '||
	 '       appr_exception    = 0, '||
	 '       approval_date     = NULL, '||
	 '       approved_by       = NULL';
--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

--Getting the new keys and updating the sequence table
n_sqlnum := 1230;
v_sql := 'SELECT COUNT(*) FROM maxdata.pm_to_be_inserted_'||v_sessionid;

EXECUTE IMMEDIATE v_sql INTO v_pogcount;
n_sqlnum := 1240;
t_text := 'v_pogcount='||v_pogcount||', v_to_pogmaster_id='||v_to_pog_master_id;
--dbms_output.put_line(t_text);

-- Can't grab key if no pogs
IF v_pogcount!= 0 THEN
	maxapp.p_get_next_key(23, 4096, v_pogcount, v_to_pog_master_id, t_error_msg);
END IF;

IF t_error_msg IS NOT NULL THEN
        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        raise_application_error (-20001, RTRIM(t_error_msg));
END IF;

n_sqlnum := 1300;
t_text   := 'Done gathering pogmaster records, new keys and updating sequence table';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('v_to_pog_master_id: ' || convert(char,v_to_pog_master_id)
--dbms_output.put_line('v_to_7mlv7loc_id: ' || convert(char,v_to_7mlv7loc_id)
--dbms_output.put_line('v_to_8llv8loc_id: ' || convert(char,v_to_8llv8loc_id)
--dbms_output.put_line('v_to_9llv9loc_id: ' || convert(char,v_to_9llv9loc_id)
--dbms_output.put_line('v_to_10llv10loc_id: ' || convert(char,v_to_10llv10loc_id)
--dbms_output.put_line('pm_to_be_inserted'
SELECT * FROM maxdata.pm_to_be_inserted
*/

-------------------------------------------------------------------
-- Updating the version id for the copied pog.
-- It will be one more than the max version for the lv7loc_userid
-- of the pog record being inserted.
-------------------------------------------------------------------

v_counter      := 0;
n_sqlnum       := 1310;

OPEN c_lv7loc;
FETCH c_lv7loc INTO v_version_id,v_lv7loc_userid,v_lv7loc_id;

v_prevlv7loc_userid := v_lv7loc_userid;
v_currlv7loc_userid := v_prevlv7loc_userid;
v_quoted_currlv7loc_userid := replace(v_currlv7loc_userid, '''', '''''');

-- The goal here is to set the version_IDs to the following structure
-- UserID: 5	Version_ID: 1
--                          2
--                          3
--         6                1
--                          2
-- etc...
n_sqlnum := 1320;

t_text := 'Start lv7loc loop';
--dbms_output.put_line(t_text);

WHILE c_lv7loc%FOUND LOOP
	--dbms_output.put_line('Current lv7loc_id='||v_lv7loc_id);
	v_currlv7loc_userid := v_lv7loc_userid;

	-- If the userid has changed then reset the counter to start a new series
	-- else increment the counter to continue the current series
	IF v_currlv7loc_userid != v_prevlv7loc_userid THEN
		v_counter := 1;
		--dbms_output.put_line('New series is '||v_currlv7loc_userid);
		--dbms_output.put_line('Prev series was '||v_prevlv7loc_userid);
		--dbms_output.put_line('Counter='||v_counter);
		v_prevlv7loc_userid := v_currlv7loc_userid;
		v_quoted_currlv7loc_userid := replace(v_currlv7loc_userid, '''', '''''');
	ELSE
		v_counter := v_counter + 1;
		--dbms_output.put_line('Counter='||v_counter);
	END IF;

	v_version_id := NVL((v_version_id),1) + v_counter;

	--dbms_output.put_line('Version_id='||v_version_id);
	v_sql := 'UPDATE maxdata.lv7loc_to_be_copied_'||v_sessionid||
		 '   SET version_id = '||v_version_id||
		 ' WHERE lv7loc_userid = '''||v_quoted_currlv7loc_userid||''''||
		 '   AND lv7loc_id     = '||v_lv7loc_id;
	--dbms_output.put_line(v_sql);
	EXECUTE IMMEDIATE (v_sql);

	COMMIT;

	FETCH c_lv7loc INTO v_version_id,v_lv7loc_userid,v_lv7loc_id;
END LOOP;

CLOSE c_lv7loc;

n_sqlnum := 1400;
t_text   := 'Done updating version ID for copied planograms';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);
--dbms_output.put_line(t_text);

-------------------------------------------------------------------
-- Updating the version id of the pogmaster for the model
-------------------------------------------------------------------

n_sqlnum := 1410;
v_sql :=
'UPDATE maxdata.pm_to_be_inserted_'||v_sessionid||
'   SET lv7_version_id = (SELECT version_id '||
'   			   FROM maxdata.lv7loc_to_be_copied_'||v_sessionid||' l7l '||
'   			  WHERE pm_to_be_inserted_'||v_sessionid||'.new_pog_model_id = l7l.Newlv7loc_id + '||v_to_7mlv7loc_id||') '||
' WHERE EXISTS (SELECT 1 '||
'		 FROM maxdata.lv7loc_to_be_copied_'||v_sessionid||' l7l '||
'	        WHERE pm_to_be_inserted_'||v_sessionid||'.new_pog_model_id = l7l.Newlv7loc_id + '||v_to_7mlv7loc_id||')';

--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('maxdata.pm_to_be_inserted after updating version id.'
SELECT * FROM maxdata.pm_to_be_inserted
*/

n_sqlnum := 1500;
t_text   := 'Done updating version ID for copied planograms in pogmaster';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

-----------------------------------------------------------------------------------
-- Update the input tables with the new IDs
-----------------------------------------------------------------------------------
n_sqlnum := 1510;
v_sql :=
'UPDATE maxdata.t_pogs_to_copy '||
'   SET new_lv7loc_id = (SELECT Newlv7loc_id + '||v_to_7mlv7loc_id||
'			   FROM maxdata.lv7loc_to_be_copied_'||v_sessionid||' B '||
'			  WHERE t_pogs_to_copy.lv7loc_id = B.lv7loc_id) '||
' WHERE t_pogs_to_copy.job_id = '||in_jobID||
'   AND EXISTS (SELECT 1 '||
'		  FROM maxdata.lv7loc_to_be_copied_'||v_sessionid||' B '||
'	         WHERE t_pogs_to_copy.lv7loc_id = B.lv7loc_id)';

--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1520;
v_sql :=
'UPDATE maxdata.t_assignments_to_copy '||
'   SET new_lv7loc_id     = (SELECT new_pog_model_id '||
'   			       FROM maxdata.pm_to_be_inserted_'||v_sessionid||' B '||
'   			      WHERE t_assignments_to_copy.pog_master_id = B.pog_master_id), '||
'       new_pog_master_id = (SELECT B.new_pog_master_id + '||v_to_pog_master_id||
'       		       FROM maxdata.pm_to_be_inserted_'||v_sessionid||' B '||
'   			      WHERE t_assignments_to_copy.pog_master_id = B.pog_master_id) '||
' WHERE t_assignments_to_copy.job_id = '||in_jobID||
'   AND EXISTS (SELECT 1 '||
'		 FROM maxdata.pm_to_be_inserted_'||v_sessionid||' B '||
'		WHERE t_assignments_to_copy.pog_master_id = B.pog_master_id)';

--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('maxdata.t_pogs_to_copy after update');
SELECT * FROM maxdata.t_pogs_to_copy
 WHERE job_id = in_jobID

--dbms_output.put_line('maxdata.t_assignments_to_copy after update');
SELECT * FROM maxdata.t_assignments_to_copy
 WHERE job_id = in_jobID
*/

n_sqlnum := 1550;
t_text   := 'Done updating input tables with new IDs';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

-----------------------------------------------------------------------------------
-- Now that we have collected all info lets insert the values into the 5 tables
-----------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------
-- Other than lv7loc_id, version_id and pog_status (P) the rest are identical to the original
---------------------------------------------------------------------------------------------
n_sqlnum := 1555;
v_sql := 'UPDATE maxdata.lv7loc_to_be_copied_'||v_sessionid||
	 '   SET lv7loc_id = Newlv7loc_id + '|| v_to_7mlv7loc_id;
--dbms_output.put_line(v_sql);
EXECUTE IMMEDIATE (v_sql);

/* Pl. comment after finishing the debugging
--dbms_output.put_line('lv7loc_to_be_copied');
SELECT * FROM maxdata.lv7loc_to_be_copied
*/

--v_sql := 'CREATE TABLE maxdata.l7l_field_names AS '||
--	 ' SELECT column_name name'||
--	 '   FROM user_tab_cols '||
--	 '  WHERE table_name = ''LV7LOC'' '||
--	 '  ORDER BY column_id';

--EXECUTE IMMEDIATE (v_sql);
n_sqlnum := 1560;
t_text := 'lv7loc field name loop';
--dbms_output.put_line(t_text);

v_sql      := 'INSERT INTO maxdata.lv7loc SELECT ';

--t_cur_stmt := 'SELECT name FROM maxdata.l7l_field_names';
t_cur_stmt := 'SELECT column_name name'||
	      '  FROM user_tab_cols '||
	      ' WHERE table_name = ''LV7LOC'' '||
	      ' ORDER BY column_id';

n_sqlnum := 1565;
OPEN c_colname FOR t_cur_stmt;
LOOP
    FETCH c_colname INTO v_name;
    EXIT WHEN c_colname%NOTFOUND;

    v_sql := v_sql || v_name || ', ';
END LOOP;
CLOSE c_colname;

n_sqlnum := 1570;
v_sql := SUBSTR(v_sql, 1, LENGTH(v_sql) - 2) || ' FROM maxdata.lv7loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(v_sql);

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1600;
t_text   := 'Completed lv7loc insert';
--dbms_output.put_line(t_text);

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--------------------------------------------------------------------------
-- Other than lv8loc_id and lv7loc_id the rest are identical to the original
--------------------------------------------------------------------------
v_sql := 'UPDATE maxdata.lv8loc_to_be_copied_'||v_sessionid||
	 '   SET lv7loc_id = Newlv7loc_id, '||
	 '       lv8loc_id = Newlv8loc_id + '|| v_to_8llv8loc_id;

EXECUTE IMMEDIATE (v_sql);

--v_sql := 'CREATE TABLE maxdata.l8l_field_names AS '||
--	 ' SELECT column_name name'||
--	 '   FROM user_tab_cols '||
--	 '  WHERE table_name = ''LV8LOC'' '||
--	 '  ORDER BY column_id';

--EXECUTE IMMEDIATE (v_sql);

t_text := 'lv8loc field name loop';
--dbms_output.put_line(t_text);

v_sql := 'INSERT INTO maxdata.lv8loc SELECT ';

--t_cur_stmt := 'SELECT name FROM maxdata.l8l_field_names';
t_cur_stmt := 'SELECT column_name name'||
	      '  FROM user_tab_cols '||
	      ' WHERE table_name = ''LV8LOC'' '||
	      ' ORDER BY column_id';

OPEN c_colname FOR t_cur_stmt;
LOOP
    FETCH c_colname INTO v_name;
    EXIT WHEN c_colname%NOTFOUND;

    v_sql := v_sql || v_name || ', ';
END LOOP;
CLOSE c_colname;

v_sql := SUBSTR(v_sql, 1, LENGTH(v_sql) - 2) || ' FROM maxdata.lv8loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(v_sql);

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1700;
t_text   := 'Completed lv8loc insert';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

------------------------------------------------------------------------------------
-- Other than lv9loc_id, lv8loc_id, lv7loc_id the rest are identical to the original
------------------------------------------------------------------------------------

n_sqlnum := 1710;
-- Before changing the lv9loc_id value, synch the lv9loc.peg_link_id to the new lv9loc_id value
v_sql := 'UPDATE maxdata.lv9loc_to_be_copied_'||v_sessionid||' t1 '||
	 '   SET peg_link_id = (SELECT t2.Newlv9loc_id + '|| v_to_9llv9loc_id ||
	 '                        FROM maxdata.lv9loc_to_be_copied_'||v_sessionid||' t2' ||
	 '                       WHERE t2.lv9loc_id = t1.peg_link_id) '||
	 ' WHERE EXISTS (SELECT * '||
         '		   FROM maxdata.lv9loc_to_be_copied_'||v_sessionid||' t2' ||
         ' 		  WHERE t2.lv9loc_id = t1.peg_link_id)';
EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1720;
v_sql := 'UPDATE maxdata.lv9loc_to_be_copied_'||v_sessionid||
	 '   SET lv7loc_id = Newlv7loc_id, '||
	 '       lv8loc_id = Newlv8loc_id, '||
	 '       lv9loc_id = Newlv9loc_id + '|| v_to_9llv9loc_id;

EXECUTE IMMEDIATE (v_sql);

--v_sql := 'CREATE TABLE maxdata.l9l_field_names AS '||
--	 ' SELECT column_name name'||
--	 '   FROM user_tab_cols '||
--	 '  WHERE table_name = ''LV9LOC'' '||
--	 '  ORDER BY column_id';

--EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1730;
t_text := 'lv9loc field name loop';
--dbms_output.put_line(t_text);

v_sql := 'INSERT INTO maxdata.lv9loc SELECT ';

--t_cur_stmt := 'SELECT name FROM maxdata.l9l_field_names';
t_cur_stmt := 'SELECT column_name name'||
	      '  FROM user_tab_cols '||
	      ' WHERE table_name = ''LV9LOC'' '||
	      ' ORDER BY column_id';

n_sqlnum := 1740;
OPEN c_colname FOR t_cur_stmt;
LOOP
    FETCH c_colname INTO v_name;
    EXIT WHEN c_colname%NOTFOUND;

    v_sql := v_sql || v_name || ', ';
END LOOP;
CLOSE c_colname;

v_sql := SUBSTR(v_sql,1,LENGTH(v_sql) - 2) || ' FROM maxdata.lv9loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(v_sql);

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1800;
t_text   := 'Completed lv9loc insert';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

------------------------------------------------------------------------------------------------
-- Other than lv10loc_id, lv9loc_id, lv8loc_id, lv7loc_id the rest are identical to the original
------------------------------------------------------------------------------------------------
n_sqlnum := 1810;
v_sql := 'UPDATE maxdata.lv10loc_to_be_copied_'||v_sessionid||
         '   SET lv7loc_id  = Newlv7loc_id, '||
         '       lv8loc_id  = Newlv8loc_id, '||
         '       lv9loc_id  = Newlv9loc_id, '||
         '       lv10loc_id = Newlv10loc_id + ' || v_to_10llv10loc_id;

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1820;
--v_sql := 'CREATE TABLE maxdata.l10l_field_names AS '||
--	 ' SELECT column_name name'||
--	 '   FROM user_tab_cols '||
--	 '  WHERE table_name = ''LV10LOC'' '||
--	 '  ORDER BY column_id';

--EXECUTE IMMEDIATE (v_sql);

t_text := 'lv10loc field name loop';
--dbms_output.put_line(t_text);

v_sql := 'INSERT INTO maxdata.lv10loc SELECT ';

--t_cur_stmt := 'SELECT name FROM maxdata.l10l_field_names';
t_cur_stmt := 'SELECT column_name name'||
	      '  FROM user_tab_cols '||
	      ' WHERE table_name = ''LV10LOC'' '||
	      ' ORDER BY column_id';

OPEN c_colname FOR t_cur_stmt;
LOOP
    FETCH c_colname INTO v_name;
    EXIT WHEN c_colname%NOTFOUND;

    v_sql := v_sql || v_name || ', ';
END LOOP;
CLOSE c_colname;

v_sql := SUBSTR(v_sql,1,LENGTH(v_sql) - 2) || ' FROM maxdata.lv10loc_to_be_copied_'||v_sessionid;
--dbms_output.put_line(v_sql);

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1900;
t_text   := 'Completed lv10loc insert';
--dbms_output.put_line(t_text);

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

-----------------------------------------------------------------------------------
-- pog_master_id (new ID), pog_model_id (the newly created model POG),
-- pog_planned_start (effective_date), pog_actual_start (effective_date),
-- pog_end_date (null), pog_start_cycle (null),
-- pog_start_period (null) , pog_end_cycle (null), lv7_version_id (same as newly created model POG)
-- pog_end_period (null), approval_status (0), pog_mvmt_applied (0), manually_approved (0),
-- appr_exception (0), approval_date (null), approved_by (null)
-- the rest are identical to the original
-----------------------------------------------------------------------------------
n_sqlnum := 1910;
v_sql := 'UPDATE maxdata.pm_to_be_inserted_'||v_sessionid||
	 '   SET pog_model_id  = new_pog_model_id, '||
	 '       pog_master_id = new_pog_master_id + '|| v_to_pog_master_id;

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1920;

--v_sql := 'CREATE TABLE maxdata.pog_field_names AS '||
--	 ' SELECT column_name name'||
--	 '   FROM user_tab_cols '||
--	 '  WHERE table_name = ''POGMASTER'' '||
--	 '  ORDER BY column_id';

--EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 1930;

t_text := 'pogmaster field name loop';
--dbms_output.put_line(t_text);

v_sql := 'INSERT INTO maxdata.pogmaster SELECT ';

--t_cur_stmt := 'SELECT name FROM maxdata.pog_field_names';

t_cur_stmt := 'SELECT column_name name'||
	      '  FROM user_tab_cols '||
	      ' WHERE table_name = ''POGMASTER'' '||
	      ' ORDER BY column_id';

OPEN c_colname FOR t_cur_stmt;
LOOP
    FETCH c_colname INTO v_name;
    EXIT WHEN c_colname%NOTFOUND;

    v_sql := v_sql || v_name || ', ';
END LOOP;
CLOSE c_colname;

v_sql := SUBSTR(v_sql,1,LENGTH(v_sql) - 2) || ' FROM maxdata.pm_to_be_inserted_'||v_sessionid;
--dbms_output.put_line(v_sql);

EXECUTE IMMEDIATE (v_sql);

n_sqlnum := 2000;
t_text   := 'Completed pogmaster insert';
--dbms_output.put_line(t_text);

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


-----------------------------------------------------------------------------------
-- Delete temp tables; use a seperate block to protect against errors
-----------------------------------------------------------------------------------
COMMIT;

BEGIN
	t_text   := 'Dropping temp tables';
	--dbms_output.put_line(t_text);

	EXECUTE IMMEDIATE ('DROP TABLE maxdata.pm_to_be_inserted_'||v_sessionid);
	EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv10loc_to_be_copied_'||v_sessionid);
	EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv9loc_to_be_copied_'||v_sessionid);
	EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv8loc_to_be_copied_'||v_sessionid);
	EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv7loc_to_be_copied_'||v_sessionid);

EXCEPTION
	-- Don't want failed table drop to kill process
	WHEN OTHERS THEN NULL;
END;

n_sqlnum := 2100;
t_text   := 'Finished copying and assigning POGs';

maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

--dbms_output.put_line(t_text);

COMMIT;

EXCEPTION
    WHEN OTHERS THEN
        -- May have been sent here with an error msg already so don't lose it
        IF t_error_msg IS NULL THEN
                t_error_msg := SQLERRM;
	END IF;

        COMMIT;

        -----------------------------------------------------------------------------------
	-- Delete temp tables; use a seperate block to protect against errors
	-----------------------------------------------------------------------------------

        BEGIN
		t_text := 'ERROR BLOCK: Dropping temp tables';
		--dbms_output.put_line(t_text);

		EXECUTE IMMEDIATE ('DROP TABLE maxdata.pm_to_be_inserted_'||v_sessionid);
		EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv10loc_to_be_copied_'||v_sessionid);
		EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv9loc_to_be_copied_'||v_sessionid);
		EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv8loc_to_be_copied_'||v_sessionid);
		EXECUTE IMMEDIATE ('DROP TABLE maxdata.lv7loc_to_be_copied_'||v_sessionid);

	EXCEPTION
		-- Don't want failed table drop to kill process
		WHEN OTHERS THEN NULL;
	END;

--        --dbms_output.put_line('END p_version_copy_pog (failed, see maxdata.auv_to_log)');
--        --dbms_output.put_line(t_error_msg);

        maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);

        RAISE;

END p_version_copy_pog;
/

GRANT EXECUTE ON maxdata.p_version_copy_pog TO maxuser,madmax;


