
DROP PROCEDURE maxdata.p_version_copy_pog
@

CREATE PROCEDURE maxdata.p_version_copy_pog
        (IN in_jobID INTEGER)

LANGUAGE SQL
P1: BEGIN

/*
Revision 1.1.2.2.2.3  2012/07/24 22:16:51  gacard
FIXID S0831894: NF: Option to include unplaced products upon planogram copy

Revision 1.1.2.2.2.2  2010/02/22 22:16:51  gacard
FIXID S0590015: NF: Planogram Maintenance needs option for handling signage

Revision 1.1.2.2.2.1  2009/07/21 19:15:21  makirk
Fix for defects S0589025 and S0576852

Revision 1.1.2.2.4.1  2009/07/09 21:01:37  makirk
FIXID S0589025: Fix for S0589025

Revision 1.1.2.2  2008/05/15 21:59:16  makirk
Fixed cursor handler in insert stmt build sections.  Fixed debugging stmts during temp table loads.

Revision 1.1.2.1  2008/05/12 21:13:12  vejang
511 files added

Revision 1.1  2008/05/09 22:22:54  makirk
Renamed to 6.x standard
Added for rename from 060_511_140_p_version_copy_pog.sql.
See originally named file for history prior to the rename.

Revision 1.4  2008/05/07 20:34:58  makirk
Removed static temp table creates and static inserts with dynamic select stmts.  References to maxdata changed to session for temp tbls.

Revision 1.3  2008/05/06 18:14:38  makirk
Added "where exists" clause to correlated updates

Revision 1.2  2008/04/18 20:46:56  makirk
Added peg_link_id update

Revision 1.1  2008/04/17 20:51:33  makirk
Fix for S0499002


-- MCK, 04/03/08, Fix for S0499002

History
-------
02/21/2014 - increase size of lv7loc_userid variables to match lv7loc.lv7loc_userid
12/07/2011 - update peg_link_ids before updating lv9loc_id
02/17/2010 - don't call p_get_next_key with count of zero
07/16/2009 - removed check for record_type = 'M' in order to copy composite planogram
04/03/2008 - Added for Planogram Maintenance

Usage
-----
In order to use this proc there are 2 tables that are populated by the calling code as follows:
        maxdata.t_pogs_to_copy that has columns job_id, lv7loc_id, new_lv7loc_id
        maxdata.t_assignments_to_copy that has columns job_id, lv7loc_id,new_lv7loc_id, pogmaster_id, new_pogmaster_id, effective_date
There are 2 tables because there could be some planograms without any assignments and some planograms will have multiple assignments.
The procedure takes one argument, in_jobID, which is used to indicate which records in the input tables are to be processed.
The procedure will make copies of all records all the corresponding records in lv7,lv8,lv9,lv10 loc and pogmaster tables.
The procedure will update the input table with the new lv7loc_ids and new pogmaster_ids.

NOTE: Dynamic SQL must be used because tables used in this proc are not permanent and therefore all SQL stmts
      using them will fail to compile
*/

-- Template vars (error handling, tracking, sql text)

-- For debugging
DECLARE t_cnt                   INTEGER;
DECLARE t_row_cnt               INTEGER;
DECLARE v_cnt                   VARCHAR(6);
DECLARE t_debug                 SMALLINT        DEFAULT 0; -- 0=debug off, 1=on

DECLARE n_sqlnum                INTEGER         DEFAULT 1000;
DECLARE t_proc_name             VARCHAR(30)     DEFAULT 'p_version_copy_pog';
DECLARE t_text                  VARCHAR(255);
DECLARE t_error_msg             VARCHAR(255);
DECLARE t_copy_unplaced         INTEGER;        --1=copy unplaced products, 0=don't copy unplaced products
DECLARE v_lv7loc_id             INTEGER;
DECLARE v_to_7mlv7loc_id        INTEGER;
DECLARE v_to_pogmaster_id       INTEGER         DEFAULT 0;
DECLARE v_to_8llv8loc_id        INTEGER         DEFAULT 0;
DECLARE v_to_9llv9loc_id        INTEGER         DEFAULT 0;
DECLARE v_to_10llv10loc_id      INTEGER         DEFAULT 0;
DECLARE v_lv10locCounter        INTEGER;
DECLARE v_lv7count              INTEGER;
DECLARE v_lv8count              INTEGER;
DECLARE v_lv9count              INTEGER;
DECLARE v_lv10count             INTEGER;
DECLARE v_pogcount              INTEGER;
DECLARE v_sql                   VARCHAR(8000);
DECLARE v_lv7loc_userid         VARCHAR(80);
DECLARE v_currlv7loc_userid     VARCHAR(80);
DECLARE v_prevlv7loc_userid     VARCHAR(80);
DECLARE v_counter               INTEGER;
DECLARE v_version_id            INTEGER;
DECLARE n_sqlcode               INTEGER;
DECLARE v_no_data               SMALLINT        DEFAULT 0;
DECLARE SQLSTATE                CHAR(5)         DEFAULT '00000';
DECLARE SQLCODE                 INTEGER;

DECLARE v_stmt                  STATEMENT;

DECLARE c_lv7loc CURSOR WITH RETURN FOR v_stmt;
DECLARE c_count  CURSOR FOR stmt;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN NOT ATOMIC
        -- Saving code/state has to be the first stmt
        -- because they are reset by any stmt (even SET stmt).

        SET t_error_msg = SQLSTATE|| ','||RTRIM(CAST(SQLCODE AS CHAR(5)));
        SET n_sqlcode   = INT(SUBSTR(t_error_msg, 7, 5));
        SET t_text      = 'error';
        SET t_error_msg = 'ERRCODE:'||t_error_msg;

        COMMIT;

        -- Log the error message.
        CALL maxdata.ins_import_log (t_proc_name, t_text, t_error_msg, v_sql, n_sqlnum, NULL);

        COMMIT;

        -- For warning, RESIGNAL returns 0 as a final return status.
        -- So, if it is warning, then change the severity.
        IF n_sqlcode >= 0 THEN
                RESIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = 'Warning returned while executing procedure (see IMPORT_LOG)';
        ELSE
                RESIGNAL;
        END IF;
END;

-- UDB cannot build temp tables dynamically.  This makes compiling/debugging easier also
DECLARE GLOBAL TEMPORARY TABLE l7l_field_names
        (name VARCHAR(128))
         WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE l8l_field_names
        (name VARCHAR(128))
         WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE l9l_field_names
        (name VARCHAR(128))
         WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE l10l_field_names
        (name VARCHAR(128))
         WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE pog_field_names
        (name VARCHAR(128))
         WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE sess_lv7loc_to_be_copied AS
        (SELECT 100000000.0 AS Newlv7loc_id,
                A.*
           FROM maxdata.lv7loc A,
                maxdata.t_pogs_to_copy B) DEFINITION ONLY
        WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE sess_lv8loc_to_be_copied AS
        (SELECT 100000000.0 AS Newlv8loc_id,
                B.Newlv7loc_id,
                A.*
           FROM maxdata.lv8loc A,
                session.sess_lv7loc_to_be_copied B) DEFINITION ONLY
        WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE sess_lv9loc_to_be_copied AS
        (SELECT 100000000.0 AS Newlv9loc_id,
                B.Newlv7loc_id,
                B.Newlv8loc_id,
                A.*
           FROM maxdata.lv9loc A,
                session.sess_lv8loc_to_be_copied B) DEFINITION ONLY
        WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE sess_lv10loc_to_be_copied AS
        (SELECT 100000000.0 AS Newlv10loc_id,
                B.Newlv7loc_id,
                100000000.0 AS Newlv8loc_id,
                100000000.0 AS Newlv9loc_id,
                A.*
          FROM maxdata.lv10loc A,
               session.sess_lv7loc_to_be_copied B) DEFINITION ONLY
        WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

DECLARE GLOBAL TEMPORARY TABLE sess_pogmaster_to_be_inserted AS
        (SELECT 100000000.0    AS new_pogmaster_id,
                C.Newlv7loc_id AS new_pog_model_id,
                A.*
           FROM maxdata.pogmaster A
           JOIN maxdata.t_assignments_to_copy B    ON A.pog_master_id = B.pog_master_id
           JOIN session.sess_lv7loc_to_be_copied C ON B.lv7loc_id     = C.lv7loc_id) DEFINITION ONLY
        WITH REPLACE NOT LOGGED ON COMMIT PRESERVE ROWS;

-- Log start
SET n_sqlnum = 1050;
SET t_text   = 'Started';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

-- Check to see if job_id is valid
SET n_sqlnum = 1052;
SELECT COUNT(*) INTO v_counter FROM maxdata.t_pogs_to_copy WHERE job_id = CAST(in_jobID AS DECIMAL(10));

SET n_sqlnum = 1054;
IF v_counter = 0 THEN
        SET t_error_msg = 'Job ID '||RTRIM(CAST(in_jobid AS CHAR(20)))||' was not found in maxdata.t_pogs_to_copy';
        CALL maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        SIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = t_error_msg;
END IF;

-------------------------------------------------------------------
-- Get the lv7loc records to be copied for the pog
-------------------------------------------------------------------
SET n_sqlnum = 1056;
INSERT INTO session.sess_lv7loc_to_be_copied
SELECT (ROW_NUMBER() OVER(ORDER BY A.lv7loc_userid))-1 AS Newlv7loc_id,
       A.*
  FROM maxdata.lv7loc A,
       maxdata.t_pogs_to_copy B
 WHERE A.lv7loc_id   = B.lv7loc_id
   AND B.job_id      = CAST(in_jobID AS DECIMAL(10));

-- Getting the new keys and updating the sequence table
SET n_sqlnum = 1058;
SELECT COUNT(*) INTO v_lv7count FROM session.sess_lv7loc_to_be_copied;

SET n_sqlnum = 1060;

IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(v_lv7count AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'Records inserted into session.sess_lv7loc_to_be_copied', v_cnt, n_sqlnum, NULL);
END IF;

IF v_lv7count != 0 THEN
    CALL maxapp.p_get_next_key(7, 2, v_lv7count, v_to_7mlv7loc_id, t_error_msg);

    IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(v_to_7mlv7loc_id AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'v_to_7mlv7loc_id=', v_cnt, n_sqlnum, NULL);
    END IF;

    IF t_error_msg IS NOT NULL THEN
        CALL maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        SIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = t_error_msg;
    END IF;
END IF;

SET n_sqlnum = 1100;
SET t_text   = 'Copying ' || RTRIM(CAST(v_lv7count AS CHAR(20))) || ' planograms(s)';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

-------------------------------------------------------------------
-- Get the lv8loc records to be copied for the lv7loc records
-------------------------------------------------------------------

SET n_sqlnum = 1102;
INSERT INTO session.sess_lv8loc_to_be_copied
SELECT (ROW_NUMBER() OVER(ORDER BY A.lv8loc_userid))-1,
       B.Newlv7loc_id + v_to_7mlv7loc_id,
       A.*
  FROM maxdata.lv8loc A,
       session.sess_lv7loc_to_be_copied B
 WHERE A.lv7loc_id = B.lv7loc_id;

/* Pl. comment after finishing the debugging
SELECT * FROM session.sess_lv8loc_to_be_copied
*/

-- Getting the new keys and updating the sequence table
SET n_sqlnum = 1104;
SELECT COUNT(*) INTO v_lv8count FROM session.sess_lv8loc_to_be_copied;

SET n_sqlnum = 1108;

IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(v_lv8count AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'Records inserted into session.sess_lv8loc_to_be_copied', v_cnt, n_sqlnum, NULL);
END IF;

IF v_lv8count != 0 THEN
    CALL maxapp.p_get_next_key(8, 2, v_lv8count, v_to_8llv8loc_id, t_error_msg);

    IF t_error_msg IS NOT NULL THEN
        CALL maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        SIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = t_error_msg;
    END IF;
END IF;

SET n_sqlnum = 1110;

-------------------------------------------------------------------
-- Get the lv9loc records to be copied for the lv8loc records
-------------------------------------------------------------------
INSERT INTO session.sess_lv9loc_to_be_copied
SELECT (ROW_NUMBER() OVER(ORDER BY A.lv9loc_userid))-1,
       B.Newlv7loc_id,
       B.Newlv8loc_id + v_to_8llv8loc_id,
       A.*
  FROM maxdata.lv9loc A,
       session.sess_lv8loc_to_be_copied B
 WHERE A.lv8loc_id = B.lv8loc_id;

/* Pl. comment after finishing the debugging
SELECT * FROM session.sess_lv9loc_to_be_copied
*/

-- Getting the new keys and updating the sequence table
SELECT COUNT(*) INTO v_lv9count FROM session.sess_lv9loc_to_be_copied;

IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(v_lv9count AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'Records inserted into session.sess_lv9loc_to_be_copied', v_cnt, n_sqlnum, NULL);
END IF;

IF v_lv9count != 0 THEN
    CALL maxapp.p_get_next_key(9, 2, v_lv9count, v_to_9llv9loc_id, t_error_msg);

    IF t_error_msg IS NOT NULL THEN
        CALL maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        SIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = t_error_msg;
    END IF;
END IF;

SET n_sqlnum = 1120;

SELECT INT(value_1) INTO t_copy_unplaced 
FROM maxapp.userpref WHERE key_1 = 'INCLUDE_UNPLACED_UPON_POG_COPY';

SET n_sqlnum = 1121;
-------------------------------------------------------------------
-- Get the lv10loc records to be copied for the lv7loc records
-- because we want products attached to fixture level shelves as well
-- as those directly attached to the planogram.
-------------------------------------------------------------------
IF t_copy_unplaced = 1 THEN
    INSERT INTO session.sess_lv10loc_to_be_copied
    SELECT COALESCE((ROW_NUMBER() OVER(ORDER BY A.lv10loc_userid))-1,-1),
           COALESCE(B.Newlv7loc_id + v_to_7mlv7loc_id,-1),
           COALESCE(A.lv8loc_id,-1),
           COALESCE(A.lv9loc_id,-1),
           A.*
      FROM maxdata.lv10loc A,
           session.sess_lv7loc_to_be_copied B
     WHERE A.lv7loc_id = B.lv7loc_id;
ELSE
    INSERT INTO session.sess_lv10loc_to_be_copied
    SELECT COALESCE((ROW_NUMBER() OVER(ORDER BY A.lv10loc_userid))-1,-1),
           COALESCE(B.Newlv7loc_id + v_to_7mlv7loc_id,-1),
           COALESCE(A.lv8loc_id,-1),
           COALESCE(A.lv9loc_id,-1),
           A.*
      FROM maxdata.lv10loc A,
           session.sess_lv7loc_to_be_copied B
     WHERE A.lv7loc_id = B.lv7loc_id
       AND A.placed = 1;
END IF;

SET n_sqlnum = 1130;
UPDATE session.sess_lv10loc_to_be_copied
   SET Newlv8loc_id = COALESCE(
                      (SELECT L8.Newlv8loc_id + v_to_8llv8loc_id
                         FROM session.sess_lv8loc_to_be_copied L8
                        WHERE session.sess_lv10loc_to_be_copied.lv8loc_id = L8.lv8loc_id),-1),
       Newlv9loc_id = COALESCE(
                      (SELECT L9.Newlv9loc_id + v_to_9llv9loc_id
                         FROM session.sess_lv9loc_to_be_copied L9
                        WHERE session.sess_lv10loc_to_be_copied.lv9loc_id = L9.lv9loc_id),-1);

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'Records in session.sess_lv10loc_to_be_copied updated', v_cnt, n_sqlnum, NULL);
END IF;

/* Pl. comment after finishing the debugging
SELECT * FROM session.sess_lv10loc_to_be_copied
*/

-- Getting the new keys and updating the sequence table
SET n_sqlnum = 1140;
SELECT COUNT(*) INTO v_lv10count FROM session.sess_lv10loc_to_be_copied;

IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(v_lv10count AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'Records inserted into session.sess_lv10loc_to_be_copied', v_cnt, n_sqlnum, NULL);
END IF;

IF v_lv10count != 0 THEN
    CALL maxapp.p_get_next_key(10, 2, v_lv10count, v_to_10llv10loc_id, t_error_msg);

    --SET t_text = 'v_lv10count='||CAST(v_lv10count AS CHAR)||', v_to_10llv10loc_id='||CAST(v_to_10llv10loc_id AS CHAR);
    --CALL maxdata.ins_import_log(t_proc_name, 'xyz', t_text, NULL, n_sqlnum, NULL);

    IF t_error_msg IS NOT NULL THEN
        CALL maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        SIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = t_error_msg;
    END IF;
END IF;

SET n_sqlnum = 1200;
SET t_text   = 'Done gathering location records, new keys and updating sequence table';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


--------------------------------------------------------------------------------
-- Get the records that need to be inserted into pogmaster
-- Taking care of assigning the copied model POG to the live location from which
-- the previous assigned POG was copied.
-- The live_lv7loc_id is being taken as is from the
-- previous pogmaster entry so, that should take care
-- of the live location assignment.
-- The new Model POG is being updated.
--------------------------------------------------------------------------------
INSERT INTO session.sess_pogmaster_to_be_inserted
SELECT (ROW_NUMBER() OVER(ORDER BY A.pog_master_id))-1,
       C.Newlv7loc_id + v_to_7mlv7loc_id,
       A.*
  FROM maxdata.pogmaster A
  JOIN maxdata.t_assignments_to_copy    B ON A.pog_master_id = B.pog_master_id
  JOIN session.sess_lv7loc_to_be_copied C ON B.lv7loc_id     = C.lv7loc_id
 WHERE B.job_id = in_jobID;

UPDATE session.sess_pogmaster_to_be_inserted
   SET pog_planned_start = (SELECT effective_date
                              FROM maxdata.t_assignments_to_copy pa
                             WHERE session.sess_pogmaster_to_be_inserted.pog_master_id = pa.pog_master_id
                               AND pa.job_id = in_jobID),
       pog_actual_start  = (SELECT effective_date
                              FROM maxdata.t_assignments_to_copy pa
                             WHERE session.sess_pogmaster_to_be_inserted.pog_master_id = pa.pog_master_id
                               AND pa.job_id = in_jobID),
       pog_end_date      = NULL,
       pog_start_cycle   = NULL,
       pog_start_period  = NULL,
       pog_end_cycle     = NULL,
       pog_end_period    = NULL,
       approval_status   = 0,
       pog_mvmt_applied  = 0,
       manually_approved = 0,
       appr_exception    = 0,
       approval_date     = NULL,
       approved_by       = NULL;

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_pogmaster_to_be_inserted records updated', v_cnt, n_sqlnum, NULL);
END IF;

-- Getting the new keys and updating the sequence table
SELECT COUNT(*) INTO v_pogcount FROM session.sess_pogmaster_to_be_inserted;

IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(v_pogcount AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'Records inserted into session.sess_pogmaster_to_be_inserted', v_cnt, n_sqlnum, NULL);
END IF;

-- Can't grab key if no pogs
SET n_sqlnum = 1210;
IF v_pogcount!= 0 THEN
        CALL maxapp.p_get_next_key(23, 4096, v_pogcount, v_to_pogmaster_id, t_error_msg);
END IF;

IF t_error_msg IS NOT NULL THEN
        CALL maxdata.ins_import_log(t_proc_name, 'error', t_error_msg, NULL, n_sqlnum, NULL);
        SIGNAL SQLSTATE '38J01' SET MESSAGE_TEXT = t_error_msg;
END IF;

SET n_sqlnum = 1300;
SET t_text   = 'Done gathering pogmaster records, new keys and updating sequence table';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


/* Pl. comment after finishing the debugging
SELECT * FROM session.sess_pogmaster_to_be_inserted
*/

-------------------------------------------------------------------
-- Updating the version id for the copied pog.
-- It will be one more than the max version for the lv7loc_userid
-- of the pog record being inserted.
-------------------------------------------------------------------

SET v_counter = 0;
SET t_cnt     = 0; -- debugging


-- The goal here is to set the version_IDs to the following structure
-- UserID: 5    Version_ID: 1
--                          2
--                          3
--         6                1
--                          2
-- etc...

BEGIN
        DECLARE c_lv7loc CURSOR FOR
        SELECT DISTINCT
               F.version_id,
               D.lv7loc_userid,
               D.lv7loc_id
          FROM (SELECT (ROW_NUMBER() OVER(ORDER BY lv7loc_userid))-1 AS Newlv7loc_id,
                       A.*
                  FROM maxdata.lv7loc A,
                       maxdata.t_pogs_to_copy B
                 WHERE A.lv7loc_id   = B.lv7loc_id
                   AND B.job_id      = in_jobid) B,
                maxdata.lv7loc D,
               (SELECT MAX(version_id) version_id,
                       lv7loc_userid
                  FROM maxdata.lv7loc
                 GROUP BY lv7loc_userid) F
         WHERE D.lv7loc_id     = B.lv7loc_id
           AND D.lv7loc_userid = F.lv7loc_userid
         ORDER BY D.lv7loc_userid,D.lv7loc_id;

        OPEN c_lv7loc;

        FETCH FROM c_lv7loc INTO v_version_id,v_lv7loc_userid,v_lv7loc_id;

        SET v_prevlv7loc_userid = v_lv7loc_userid;
        SET v_currlv7loc_userid = v_prevlv7loc_userid;
        WHILE (SQLCODE = 0) DO
                SET v_currlv7loc_userid = v_lv7loc_userid;

                -- If the userid has changed then reset the counter to start a new series
                -- else increment the counter to continue the current series
                IF v_currlv7loc_userid != v_prevlv7loc_userid THEN
                        SET v_counter = 1;
                        SET v_prevlv7loc_userid = v_currlv7loc_userid;
                ELSE
                        SET v_counter = v_counter + 1;
                END IF;

                SET v_version_id = COALESCE((v_version_id),1) + v_counter;

                -- Update the current record via cursor pointer
                UPDATE session.sess_lv7loc_to_be_copied
                   SET version_id = v_version_id
                 WHERE lv7loc_userid = v_lv7loc_userid
                   AND lv7loc_id     = v_lv7loc_id;

                IF t_debug != 0 THEN
                        GET DIAGNOSTICS t_row_cnt = row_count;
                        SET t_cnt = t_cnt + t_row_cnt;
                END IF;

FETCH FROM c_lv7loc INTO v_version_id,v_lv7loc_userid,v_lv7loc_id;
        END WHILE;

        CLOSE c_lv7loc;
END;

IF t_debug != 0 THEN
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_lv7loc_to_be_copied versions updated (cursor)', v_cnt, n_sqlnum, NULL);
END IF;

SET n_sqlnum = 1400;
SET t_text   = 'Done updating version ID for copied planograms';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


-------------------------------------------------------------------
-- Updating the version id of the pogmaster for the model
-------------------------------------------------------------------
UPDATE session.sess_pogmaster_to_be_inserted
   SET lv7_version_id = (SELECT version_id
                           FROM session.sess_lv7loc_to_be_copied l7l
                          WHERE session.sess_pogmaster_to_be_inserted.new_pog_model_id = l7l.Newlv7loc_id + v_to_7mlv7loc_id)
 WHERE EXISTS (SELECT 1
                 FROM session.sess_lv7loc_to_be_copied l7l
                WHERE session.sess_pogmaster_to_be_inserted.new_pog_model_id = l7l.Newlv7loc_id + v_to_7mlv7loc_id);

/* Pl. comment after finishing the debugging
SELECT * FROM session.sess_pogmaster_to_be_inserted
*/

SET n_sqlnum = 1500;
SET t_text   = 'Done updating version ID for copied planograms in pogmaster';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


-----------------------------------------------------------------------------------
-- Update the input tables with the new IDs
-----------------------------------------------------------------------------------
UPDATE maxdata.t_pogs_to_copy
   SET new_lv7loc_id = (SELECT Newlv7loc_id + v_to_7mlv7loc_id
                          FROM session.sess_lv7loc_to_be_copied B
                         WHERE t_pogs_to_copy.lv7loc_id = B.lv7loc_id)
 WHERE job_id = in_jobID
   AND EXISTS (SELECT 1
                 FROM session.sess_lv7loc_to_be_copied B
                WHERE t_pogs_to_copy.lv7loc_id = B.lv7loc_id);

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'maxdata.t_pogs_to_copy records updated', v_cnt, n_sqlnum, NULL);
END IF;

SET n_sqlnum = 1510;
UPDATE maxdata.t_assignments_to_copy
   SET new_lv7loc_id    = (SELECT new_pog_model_id
                             FROM session.sess_pogmaster_to_be_inserted B
                            WHERE t_assignments_to_copy.pog_master_id = B.pog_master_id),
       new_pog_master_id = (SELECT B.new_pogmaster_id + v_to_pogmaster_id
                              FROM session.sess_pogmaster_to_be_inserted B
                             WHERE t_assignments_to_copy.pog_master_id = B.pog_master_id)
 WHERE job_id = in_jobID
   AND EXISTS (SELECT 1
                 FROM session.sess_pogmaster_to_be_inserted B
                WHERE t_assignments_to_copy.pog_master_id = B.pog_master_id);

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'maxdata.t_assignments_to_copy records updated', v_cnt, n_sqlnum, NULL);
END IF;

-- Pl. comment after finishing the debugging
--SELECT * FROM maxdata.t_pogs_to_copy
-- WHERE job_id = in_jobID

--SELECT * FROM maxdata.t_assignments_to_copy
-- WHERE job_id = in_jobID

SET n_sqlnum = 1550;
SET t_text   = 'Done updating input tables with new IDs';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


-------------------------------------------------------------------------------
-- Now that we have collected all info lets insert the values into the 5 tables
-------------------------------------------------------------------------------
SET n_sqlnum = 1552;

---------------------------------------------------------------------------------------------
-- Other than lv7loc_id, version_id and pog_status (P) the rest are identical to the original
---------------------------------------------------------------------------------------------
UPDATE session.sess_lv7loc_to_be_copied
   SET lv7loc_id = Newlv7loc_id + v_to_7mlv7loc_id;

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_lv7loc_to_be_copied records updated', v_cnt, n_sqlnum, NULL);
END IF;

/* Pl. comment after finishing the debugging
SELECT * FROM session.sess_lv7loc_to_be_copied
*/

SET n_sqlnum = 1560;

INSERT INTO session.l7l_field_names
SELECT colname
  FROM syscat.columns
 WHERE tabname   = 'LV7LOC'
   AND tabschema = 'MAXDATA'
 ORDER BY colno;

SET n_sqlnum = 1570;
SET v_sql = 'INSERT INTO maxdata.lv7loc SELECT ';

SET n_sqlnum = 1580;
FOR c_colname AS curs_colname CURSOR FOR
    SELECT name
      FROM session.l7l_field_names
DO
        SET v_sql = v_sql || name || ', ';
END FOR;

SET n_sqlnum = 1590;
SET v_sql = SUBSTR(v_sql, 1, LENGTH(v_sql) - 2) || ' FROM session.sess_lv7loc_to_be_copied';

SET n_sqlnum = 1595;
EXECUTE IMMEDIATE v_sql;

SET n_sqlnum = 1600;
SET t_text   = 'Completed lv7loc insert';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


----------------------------------------------------------------------------
-- Other than lv8loc_id and lv7loc_id the rest are identical to the original
----------------------------------------------------------------------------

UPDATE session.sess_lv8loc_to_be_copied
   SET lv7loc_id = Newlv7loc_id,
       lv8loc_id = Newlv8loc_id + v_to_8llv8loc_id;

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_lv8loc_to_be_copied records updated', v_cnt, n_sqlnum, NULL);
END IF;

SET n_sqlnum = 1610;


INSERT INTO session.l8l_field_names
SELECT colname
  FROM syscat.columns
 WHERE tabname   = 'LV8LOC'
   AND tabschema = 'MAXDATA'
 ORDER BY colno;

SET n_sqlnum = 1612;
SET v_sql = 'INSERT INTO maxdata.lv8loc SELECT ';

SET n_sqlnum = 1614;
FOR c_colname AS curs_colname CURSOR FOR
    SELECT name
      FROM session.l8l_field_names
DO
        SET v_sql = v_sql || name || ', ';
END FOR;

SET n_sqlnum = 1616;
SET v_sql = SUBSTR(v_sql, 1, LENGTH(v_sql) - 2) || ' FROM session.sess_lv8loc_to_be_copied';

SET n_sqlnum = 1618;
EXECUTE IMMEDIATE v_sql;

SET n_sqlnum = 1700;
SET t_text   = 'Completed lv8loc insert';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


------------------------------------------------------------------------------------
-- Other than lv9loc_id, lv8loc_id, lv7loc_id the rest are identical to the original
------------------------------------------------------------------------------------

-- Before changing the lv9loc_id value, synch the lv9loc.peg_link_id to the new lv9loc_id value

UPDATE session.sess_lv9loc_to_be_copied t1
   SET peg_link_id = (SELECT t2.Newlv9loc_id + v_to_9llv9loc_id
                        FROM session.sess_lv9loc_to_be_copied t2
                       WHERE t2.lv9loc_id = t1.peg_link_id)
 WHERE EXISTS (SELECT *
                 FROM session.sess_lv9loc_to_be_copied t2
                WHERE t2.lv9loc_id = t1.peg_link_id);

SET n_sqlnum = 1705;

UPDATE session.sess_lv9loc_to_be_copied
   SET lv7loc_id = Newlv7loc_id,
       lv8loc_id = Newlv8loc_id,
       lv9loc_id = Newlv9loc_id + v_to_9llv9loc_id;

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_lv9loc_to_be_copied records updated', v_cnt, n_sqlnum, NULL);
END IF;


SET n_sqlnum = 1710;


INSERT INTO session.l9l_field_names
SELECT colname
  FROM syscat.columns
 WHERE tabname   = 'LV9LOC'
   AND tabschema = 'MAXDATA'
 ORDER BY colno;

SET v_sql = 'INSERT INTO maxdata.lv9loc SELECT ';

FOR c_colname AS curs_colname CURSOR FOR
    SELECT name
      FROM session.l9l_field_names
DO
        SET v_sql = v_sql || name || ', ';
END FOR;

SET v_sql = SUBSTR(v_sql,1,LENGTH(v_sql) - 2) || ' FROM session.sess_lv9loc_to_be_copied';

EXECUTE IMMEDIATE v_sql;

SET n_sqlnum = 1800;
SET t_text   = 'Completed lv9loc insert';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

------------------------------------------------------------------------------------------------
-- Other than lv10loc_id, lv9loc_id, lv8loc_id, lv7loc_id the rest are identical to the original
------------------------------------------------------------------------------------------------

-- Not null
UPDATE session.sess_lv10loc_to_be_copied
   SET lv7loc_id  = Newlv7loc_id
 WHERE Newlv7loc_id != -1;

UPDATE session.sess_lv10loc_to_be_copied
   SET lv8loc_id  = Newlv8loc_id
 WHERE Newlv8loc_id != -1;

 UPDATE session.sess_lv10loc_to_be_copied
    SET lv9loc_id = Newlv9loc_id
  WHERE Newlv9loc_id != -1;

 UPDATE session.sess_lv10loc_to_be_copied
    SET lv10loc_id = Newlv10loc_id + v_to_10llv10loc_id
  WHERE Newlv10loc_id != -1;

-- Null
UPDATE session.sess_lv10loc_to_be_copied
   SET lv7loc_id  = NULL
 WHERE Newlv7loc_id = -1;

UPDATE session.sess_lv10loc_to_be_copied
   SET lv8loc_id  = NULL
 WHERE Newlv8loc_id = -1;

 UPDATE session.sess_lv10loc_to_be_copied
    SET lv9loc_id = NULL
  WHERE Newlv9loc_id = -1;

 UPDATE session.sess_lv10loc_to_be_copied
    SET lv10loc_id = v_to_10llv10loc_id
  WHERE Newlv10loc_id = -1;

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_lv10loc_to_be_copied records updated', v_cnt, n_sqlnum, NULL);
END IF;

SET n_sqlnum = 1810;

INSERT INTO session.l10l_field_names
SELECT colname
  FROM syscat.columns
 WHERE tabname   = 'LV10LOC'
   AND tabschema = 'MAXDATA'
 ORDER BY colno;

SET v_sql = 'INSERT INTO maxdata.lv10loc SELECT ';

FOR c_colname AS curs_colname CURSOR FOR
        SELECT name
          FROM session.l10l_field_names
DO
        SET v_sql = v_sql || name || ', ';
END FOR;

SET v_sql = SUBSTR(v_sql,1,LENGTH(v_sql) - 2) || ' FROM session.sess_lv10loc_to_be_copied';

EXECUTE IMMEDIATE v_sql;

SET n_sqlnum = 1900;
SET t_text   = 'Completed lv10loc insert';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);


-----------------------------------------------------------------------------------
-- pogmaster_id (new ID), pog_model_id (the newly created model POG),
-- pog_planned_start (effective_date), pog_actual_start (effective_date),
-- pog_end_date (null), pog_start_cycle (null),
-- pog_start_period (null) , pog_end_cycle (null), lv7_version_id (same as newly created model POG)
-- pog_end_period (null), approval_status (0), pog_mvmt_applied (0), manually_approved (0),
-- appr_exception (0), approval_date (null), approved_by (null)
-- the rest are identical to the original
-----------------------------------------------------------------------------------

UPDATE session.sess_pogmaster_to_be_inserted
   SET pog_model_id  = new_pog_model_id,
       pog_master_id = new_pogmaster_id + v_to_pogmaster_id;

IF t_debug != 0 THEN
        GET DIAGNOSTICS t_cnt = row_count;
        SET v_cnt = RTRIM(CAST(t_cnt AS CHAR(10)));
        CALL maxdata.ins_import_log(t_proc_name, 'dbg', 'session.sess_pogmaster_to_be_inserted records updated', v_cnt, n_sqlnum, NULL);
END IF;

SET n_sqlnum = 1910;

INSERT INTO session.pog_field_names
SELECT colname
  FROM syscat.columns
 WHERE tabname   = 'POGMASTER'
   AND tabschema = 'MAXDATA'
 ORDER BY colno;

SET v_sql = 'INSERT INTO maxdata.pogmaster SELECT ';

FOR c_colname AS curs_colname CURSOR FOR
    SELECT name
      FROM session.pog_field_names
DO
        SET v_sql = v_sql || name || ', ';
END FOR;

SET v_sql = SUBSTR(v_sql,1,LENGTH(v_sql) - 2) || ' FROM session.sess_pogmaster_to_be_inserted';

EXECUTE IMMEDIATE v_sql;

SET n_sqlnum = 2000;
SET t_text   = 'Completed pogmaster insert';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

SET n_sqlnum = 2100;
SET t_text   = 'Finished copying and assigning POGs';

CALL maxdata.ins_import_log(t_proc_name, 'info', t_text, NULL, n_sqlnum, NULL);

COMMIT;

END P1
@

GRANT EXECUTE ON PROCEDURE maxdata.p_version_copy_pog TO maxuser,madmax
@

